<?php
namespace NomadyCore\Widgets;

use Elementor\Widget_Base;
use Elementor\Controls_Manager;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

/**
 * Elementor Hello World
 *
 * Elementor widget for hello world.
 *
 * @since 1.0.0
 */
class Block_Heading extends Widget_Base {

	use \NomadyCore\Traits\Helper;

	/**
	 * Retrieve the widget name.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 * @return string Widget name.
	 */
	public function get_name() {
		return 'nomady-block-heading';
	}

	/**
	 * Retrieve the widget title.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 * @return string Widget title.
	 */
	public function get_title() {
		return __( 'Nomady Block Heading', 'nomady-core' );
	}

	/**
	 * Retrieve the widget icon.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 * @return string Widget icon.
	 */
	public function get_icon() {
		return 'eicon-animated-headline';
	}

	/**
	 * Retrieve the list of categories the widget belongs to.
	 *
	 * Used to determine where to display the widget in the editor.
	 *
	 * Note that currently Elementor supports only one category.
	 * When multiple categories passed, Elementor uses the first one.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 * @return array Widget categories.
	 */
	public function get_categories() {
		return [ 'nomady' ];
	}

	/**
	 * Retrieve the list of scripts the widget depended on.
	 *
	 * Used to set scripts dependencies required to run the widget.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 * @return array Widget scripts dependencies.
	 */
	public function get_script_depends() {
		return [ 'nomady-core' ];
	}

	/**
	 * Register the widget controls.
	 *
	 * Adds different input fields to allow the user to change and customize the widget settings.
	 *
	 * @since 1.0.0
	 *
	 * @access protected
	 */
	protected function register_controls() {
		$this->start_controls_section(
			'heading_section',
			[
				'label' => esc_html__( 'Heading', 'nomady-core' ),
			]
		);

		$this->add_control(
			'heading',
			[
				'label' => esc_html__( 'Heading', 'nomady-core' ),
				'type' => \Elementor\Controls_Manager::TEXT,
			]
		);

		$this->add_control(
			'subheading',
			[
				'label' => esc_html__( 'Subheading', 'nomady-core' ),
				'type' => \Elementor\Controls_Manager::TEXT,
			]
		);

		$this->add_control(
			'heading_url',
			[
				'label' => esc_html__( 'Heading URL', 'nomady-core' ),
				'type' => \Elementor\Controls_Manager::URL,
			]
		);

		$this->add_control(
			'heading_url_text',
			[
				'label' => esc_html__( 'Heading URL text', 'nomady-core' ),
				'type' => \Elementor\Controls_Manager::TEXT,
				'default' => esc_html__( 'View more', 'nomady-core' ),
			]
		);

		$this->end_controls_section();

		$this->start_controls_section(
			'settings_section',
			[
				'label' => esc_html__( 'Settings', 'nomady-core' ),
			]
		);

		$this->add_control(
			'heading_width',
			[
				'label' => esc_html__( 'Heading width', 'nomady-core' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'options' => [
					'default' => esc_html__( 'Default', 'nomady-core' ),
					'wide' => esc_html__( 'Wide', 'nomady-core' ),
					'fullwidth' => esc_html__( 'Full-width', 'nomady-core' ),
				],
				'default' => 'default',
			]
		);

		$this->end_controls_section();
	}

	/**
	 * Render the widget output on the frontend.
	 *
	 * Written in PHP and used to generate the final HTML.
	 *
	 * @since 1.0.0
	 *
	 * @access protected
	 */
	protected function render() {
		// Get widget settings
		$settings = $this->get_settings_for_display();

        $this->add_render_attribute(
            'section_wrapper',
            [
                'id' => 'mnmd-section-heading-' . esc_attr( $this->get_id() ),
                'class' => [
                    'mnmd-section-heading',
                ],
            ]
        );

        if ( $settings['heading'] ) {
        	switch ( $settings['heading_width']) {
        		case 'default':
        		default:
        			$container_class = 'container';
        			break;

        		case 'wide':
        			$container_class = 'container-wide';
        			break;

        		case 'fullwidth':
        			$container_class = 'container-fullwidth';
        			break;
        	}

        	echo '<div ' . $this->get_render_attribute_string( 'section_wrapper' ) . '>';
        		echo '<div class="' . esc_attr( $container_class ) . '">';

        ?>
        			<div class="mnmd-section-heading-inner d-sm-flex justify-content-between align-items-end">
		                <div class="mnmd-section-heading-primary">
		                    <h4 class="mnmd-section-heading-title h3 meta-font">
		                        <?php
		                        echo wp_kses_post( $settings['heading'] );
		                        ?>
		                    </h4>

		                    <?php
		                    if ( $settings['subheading'] ) {
		                        echo '<div class="mnmd-section-heading-subtitle">';
		                        echo wp_kses_post( $settings['subheading'] );
		                        echo '</div>';
		                    }
		                    ?>
		                </div>

		                <div class="mnmd-section-heading-secondary mt-2 mt-sm-0">
		                    <?php
		                    if ( $settings['heading_url']['url'] && $settings['heading_url_text'] ) {
		                    ?>
		                        <a class="st-btn-icon-animation-on-hover" href="<?php echo esc_url( $settings['heading_url']['url'] ); ?>"<?php
		                            if( $settings['heading_url']['is_external'] ) {
		                                if( $settings['heading_url']['nofollow'] ) {
		                                    echo ' target="_blank" rel="noopener noreferrer nofollow"';
		                                } else {
		                                    echo ' target="_blank" rel="noopener noreferrer"';
		                                }
		                            } ?>>
		                            <?php
		                            echo '<span class="align-middle">';
		                            echo wp_kses_post( $settings['heading_url_text'] );
		                            echo '</span>';
		                            echo nomady_svg_icon( array( 'icon' => 'arrow-right', 'class' => 'mnmd-icon-last' ) );
		                            ?>
		                        </a>
		                    <?php
		                    }
		                    ?>
		                </div>
		            </div>
        <?php
        		echo '</div>';
        	echo '</div>';
        }
	}
}