<?php
namespace NomadyCore\Widgets;

use Elementor\Widget_Base;
use Elementor\Controls_Manager;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

/**
 * Elementor Category Listing
 *
 * Elementor widget for Nomady Elementor plugin.
 *
 * @since 1.0.0
 */
class Category_Listing extends Widget_Base {

	use \NomadyCore\Traits\Helper;

	/**
	 * Retrieve the widget name.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 * @return string Widget name.
	 */
	public function get_name() {
		return 'nomady-category-listing';
	}

	/**
	 * Retrieve the widget title.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 * @return string Widget title.
	 */
	public function get_title() {
		return __( 'Nomady Category Listing', 'nomady-core' );
	}

	/**
	 * Retrieve the widget icon.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 * @return string Widget icon.
	 */
	public function get_icon() {
		return 'eicon-gallery-grid';
	}

	/**
	 * Retrieve the list of categories the widget belongs to.
	 *
	 * Used to determine where to display the widget in the editor.
	 *
	 * Note that currently Elementor supports only one category.
	 * When multiple categories passed, Elementor uses the first one.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 * @return array Widget categories.
	 */
	public function get_categories() {
		return [ 'nomady' ];
	}

	/**
	 * Retrieve the list of scripts the widget depended on.
	 *
	 * Used to set scripts dependencies required to run the widget.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 * @return array Widget scripts dependencies.
	 */
	public function get_script_depends() {
		return [ 'select2-selected-order' ];
	}

	/**
	 * Register the widget controls.
	 *
	 * Adds different input fields to allow the user to change and customize the widget settings.
	 *
	 * @since 1.0.0
	 *
	 * @access protected
	 */
	protected function register_controls() {
		$this->start_controls_section(
			'heading_section',
			[
				'label' => esc_html__( 'Heading', 'nomady-core' ),
			]
		);

		$this->add_control(
			'heading',
			[
				'label' => esc_html__( 'Heading', 'nomady-core' ),
				'type' => Controls_Manager::TEXT,
			]
		);

		$this->add_control(
			'subheading',
			[
				'label' => esc_html__( 'Subheading', 'nomady-core' ),
				'type' => Controls_Manager::TEXT,
			]
		);

		$this->add_control(
			'heading_url',
			[
				'label' => esc_html__( 'Heading URL', 'nomady-core' ),
				'type' => Controls_Manager::URL,
			]
		);

		$this->add_control(
			'heading_url_text',
			[
				'label' => esc_html__( 'Heading URL text', 'nomady-core' ),
				'type' => Controls_Manager::TEXT,
				'default' => esc_html__( 'View more', 'nomady-core' ),
			]
		);

		$this->end_controls_section();

		$this->start_controls_section(
            'content_section',
            [
                'label' => esc_html__( 'Content', 'nomady-core' ),
            ]
        );

        $this->add_control(
            'select_by',
            [
                'label' => esc_html__( 'Select by', 'nomady-core' ),
                'type' => \Elementor\Controls_Manager::SELECT,
                'options' => [
                    'default' => esc_html__( 'Default', 'nomady-core' ),
                    'name' => esc_html__( 'Name', 'nomady-core' ),
                    'id' => esc_html__( 'ID', 'nomady-core' ),
                ],
                'default' => 'default',
            ]
        );

        $this->add_control(
            'category_ids_name',
            [
                'label' => esc_html__( 'Categories', 'nomady-core' ),
                'type' => \Elementor\Controls_Manager::SELECT2,
                'label_block' => true,
                'multiple' => true,
                'options' => wp_list_pluck( get_terms( 'category' ), 'name', 'term_id' ),
                'condition' => [
                    'select_by' => 'name',
                ],
            ]
        );

        $this->add_control(
            'category_ids',
            [
                'label' => esc_html__( 'Category ID list', 'nomady-core' ),
                'description' => esc_html__( 'IDs of categories, separated by ",". You can check the IDs of your categories', 'nomady-core' ) . ' <a href="' . esc_url( admin_url( 'edit-tags.php?taxonomy=category' ) ) . '"> ' . esc_html__( 'here', 'nomady-core' ) . ' </a> ',
                'type' => \Elementor\Controls_Manager::TEXT,
                'condition' => [
                    'select_by' => 'id',
                ],
            ]
        );

        $this->add_control(
            'number',
            [
                'label' => esc_html__('Number of items', 'nomady-core'),
                'type' => \Elementor\Controls_Manager::NUMBER,
                'default' => 4,
                'condition' => [
                    'select_by' => 'default',
                ],
            ]
        );

        $this->add_control(
            'orderby',
            [
                'label' => esc_html__('Order by', 'nomady-core'),
                'description' => esc_html__( 'Order by ID list only available when selecting categories by IDs', 'nomady-core' ),
                'type' => \Elementor\Controls_Manager::SELECT,
                'options' => [
                    'name' => esc_html__( 'Name', 'nomady-core' ),
                    'count' => esc_html__( 'Post count', 'nomady-core' ),
                    'include' => esc_html__( 'ID list', 'nomady-core' ),
                ],
                'default' => 'name',

            ]
        );

        $this->add_control(
            'order',
            [
                'label' => esc_html__('Order', 'nomady-core'),
                'type' => \Elementor\Controls_Manager::SELECT,
                'options' => [
                    'asc' => esc_html__( 'Ascending', 'nomady-core' ),
                    'desc' => esc_html__( 'Descending', 'nomady-core' ),
                ],
                'default' => 'asc',
                'condition' => [
                    'orderby!' => 'include',
                ],

            ]
        );

        $this->end_controls_section();

		$this->start_controls_section(
			'settings_section',
			[
				'label' => esc_html__( 'Settings', 'nomady-core' ),
			]
		);

		$this->add_control(
			'layout',
			[
				'label' => esc_html__( 'Layout', 'nomady-core' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'options' => [
					'1' => esc_html__( 'Layout 1', 'nomady-core' ),
					'2' => esc_html__( 'Layout 2', 'nomady-core' ),
					'3' => esc_html__( 'Layout 3', 'nomady-core' ),
				],
				'default' => '3',
			]
		);

		$this->add_control(
            'tall_item',
            [
                'label' => __( 'Tall item', 'nomady-core' ),
                'type' => \Elementor\Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'nomady-core' ),
				'label_off' => esc_html__( 'No', 'nomady-core' ),
				'return_value' => 'yes',
				'default' => 'no',
				'condition' => [
                    'layout' => '1',
                ],
            ]
        );

		$this->add_control(
			'columns',
			[
				'label' => esc_html__( 'Columns', 'nomady-core' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'options' => [
					'3' => esc_html__( '3', 'nomady-core' ),
					'4' => esc_html__( '4', 'nomady-core' ),
				],
				'default' => '4',
				'condition' => [
                    'layout' => [
						'1',
					],
                ],
			]
		);

		$this->add_control(
            'show_post_count',
            [
                'label' => __( 'Show post count', 'nomady-core' ),
                'type' => \Elementor\Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'nomady-core' ),
				'label_off' => esc_html__( 'No', 'nomady-core' ),
				'return_value' => 'yes',
				'default' => 'yes',
            ]
        );

        $this->add_control(
			'align_center',
			[
				'label' => esc_html__( 'Align center', 'nomady-core' ),
				'type' => \Elementor\Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Yes', 'nomady-core' ),
				'label_off' => esc_html__( 'No', 'nomady-core' ),
				'return_value' => 'yes',
				'default' => 'yes',
				'condition' => [
                    'layout' => [
						'2',
						'3',
					],
                ],
			]
		);

		$this->end_controls_section();
	}

	/**
	 * Render the widget output on the frontend.
	 *
	 * Written in PHP and used to generate the final HTML.
	 *
	 * @since 1.0.0
	 *
	 * @access protected
	 */
	protected function render() {
		// Get widget settings
		$settings = $this->get_settings_for_display();

		// Create variable to pass to template.
		$template_args = array(
			'settings' => $settings,
		);

        $this->add_render_attribute(
            'section_wrapper',
            [
                'id' => 'mnmd-category-listing-' . esc_attr( $this->get_id() ),
                'class' => [
                    'mnmd-category-listing',
                    'mnmd-category-listing-' . esc_attr( $settings['layout'] ),
                ],
            ]
        );

        echo '<div ' . $this->get_render_attribute_string( 'section_wrapper' ) . '>';
        nomady_get_template_part( 'template-parts/sections/category-listing', NULL, $template_args );
        echo '</div>';
	}
}